/* AbiWord: ie_imp_HTML - plugin for non-XHTML HTML
 * Copyright (C) 2002 Francis James Franklin <fjf@alinameridon.com>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 */


// AbiWord includes

#include "ut_growbuf.h"
#include "ut_string.h"
#include "ut_assert.h"
#include "ut_debugmsg.h"

#include "pd_Document.h"

// Local includes

#include "ie_imp_HTML.h"
#include "HTML_Parser.h"
#include "HTML_tags.h"

/*****************************************************************/
/*****************************************************************/

#include "xap_Module.h"

ABI_PLUGIN_DECLARE("HTML")

// we use a reference-counted sniffer
static IE_Imp_HTML_Sniffer * m_sniffer = 0;

ABI_FAR_CALL
int abi_plugin_register (XAP_ModuleInfo * mi)
{
  if (!m_sniffer)
    {
      m_sniffer = new IE_Imp_HTML_Sniffer ();
    }
  else
    {
      m_sniffer->ref ();
    }

  mi->name = "HTML Importer";
  mi->desc = "Import HTML Documents (Experimental)";
  mi->version = ABI_VERSION_STRING;
  mi->author = "Abi the Ant";
  mi->usage = "No Usage";

  IE_Imp::registerImporter (m_sniffer);
  return 1;
}

ABI_FAR_CALL
int abi_plugin_unregister (XAP_ModuleInfo * mi)
{
  mi->name = 0;
  mi->desc = 0;
  mi->version = 0;
  mi->author = 0;
  mi->usage = 0;

  UT_ASSERT (m_sniffer);

  IE_Imp::unregisterImporter (m_sniffer);

  if (!m_sniffer->unref ())
    {
      m_sniffer = 0; // Shouldn't this be deleted?
    }

  return 1;
}

ABI_FAR_CALL
int abi_plugin_supports_version (UT_uint32 major, UT_uint32 minor, UT_uint32 release)
{
  return 1;
}

/*****************************************************************/
/*****************************************************************/

bool IE_Imp_HTML_Sniffer::recognizeContents (const char * szBuf, UT_uint32 iNumbytes)
{
  HTML_Parser parser;
  return parser.sniff (szBuf, iNumbytes);
}

bool IE_Imp_HTML_Sniffer::recognizeSuffix (const char * szSuffix)
{
  return (!(UT_stricmp (szSuffix, ".html")) || !(UT_stricmp (szSuffix, ".htm")));
}

UT_Error IE_Imp_HTML_Sniffer::constructImporter (PD_Document * pDocument, IE_Imp ** ppie)
{
  IE_Imp_HTML * p = new IE_Imp_HTML (pDocument);
  *ppie = p;
  return UT_OK;
}

bool IE_Imp_HTML_Sniffer::getDlgLabels (const char ** pszDesc, const char ** pszSuffixList, IEFileType * ft)
{
  *pszDesc = "HTML (.html, .htm)";
  *pszSuffixList = "*.html; *.htm";
  *ft = getFileType ();
  return true;
}

/*****************************************************************/
/*****************************************************************/

#ifdef __MRC__
extern "C" {
#endif

static int s_str_compare (const void * a, const void * b)
{
  const char * name = (const char *) a;
  const htmlToIdMapping * id = (const htmlToIdMapping *) b;
		
  return UT_stricmp (name, id->m_name);
}

#ifdef __MRC__
};
#endif

static int s_mapNameToToken (const char * name)
{
  size_t length = sizeof (s_Tokens) / sizeof (s_Tokens[0]);

  htmlToIdMapping * id = (htmlToIdMapping *) bsearch (name, s_Tokens, (int) length, sizeof (htmlToIdMapping), s_str_compare);

  if (id) return id->m_type;
  return TT_OTHER;
}

/*****************************************************************/
/*****************************************************************/

void IE_Imp_HTML::startElement (const char * name, const char ** atts)
{
  switch (s_mapNameToToken (name))
    {
    default:
      break;
    }
}

void IE_Imp_HTML::endElement (const char * name)
{
  switch (s_mapNameToToken (name))
    {
    default:
      break;
    }
}

void IE_Imp_HTML::charData (const char * buffer, int length)
{
  if (m_CharDataMode == cdm_Ignore) return;

  // Should we have a cdm_ParagraphDeferred; ? The idea is to eliminate <p></p> constructs...

  if (m_pGB == 0) m_pGB = new UT_GrowBuf;
  UT_decodeUTF8string (buffer, length, m_pGB);
}

void IE_Imp_HTML::flushData ()
{
  if (m_pGB == 0) return;
  if (m_pGB->getLength ())
    {
      // Add accumulated char. data to document and:
      m_pGB->truncate (0);
    }
}

/*****************************************************************/
/*****************************************************************/

IE_Imp_HTML::IE_Imp_HTML (PD_Document * pDocument) :
  IE_Imp(pDocument),
  m_CharDataMode(cdm_Ignore),
  m_pHTML(0),
  m_pGB(0)
{
  //
}

IE_Imp_HTML::~IE_Imp_HTML ()
{
  DELETEP (m_pHTML);
  DELETEP (m_pGB);
}

/*****************************************************************/
/*****************************************************************/

UT_Error IE_Imp_HTML::importFile (const char * szFilename)
{
  m_pHTML = new HTML_Parser;
  m_pHTML->setListener (this);

  m_CharDataMode = cdm_Ignore;

  UT_Error ret = UT_OK;
  if (!m_pHTML->parse (szFilename)) ret = UT_ERROR;

  DELETEP (m_pHTML);
  return ret;
}
